<?php
/**
 * Daoクラス
 * @package     Me
 * @copyright   Copyright (C) 2011 MicroEngine Inc.
 */
class Me_Dao
{
    /* 符号付き32bit Integer の最大値 */
    const INT32_MAX = 2147483647;
    /**
     * テーブル名
     * @var string
     */
    public $table_name = '';
    /**
     * Entityクラス名
     * @var string
     */
    public $class_name = '';
    /**
     * カラム定義
     * 継承したクラスで再定義する
     * @var array
     */
    public $column_list = array();

    /**
     * コンストラクタ
     */
    public function __construct()
    {

    }

    /**
     * SELECT
     * 1レコードのオブジェクトを返す
     * @param string $_id _idフィールド値
     * @param string $class_name クラス名
     * @param boolean $error_off レコードがない場合に、Exceptionにするかflaseを返すか
     * @return object $obj
     */
    public function select($_id, $class_name = null, $error_off = false)
    {
        $obj = null;
        if ($_id === null || $_id === '') {
            throw new Me_Exception(600);
        }

        // select文作成
        $query = 'SELECT ';
        // 対象カラム追加
        $query .= join(',', $this->column_list);
        $query .= "\n FROM {$this->table_name} WHERE _id = 0";

        // SQL実行
        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind(1, 5, $_id);
        $rows = $sql->exec();
        if (count($rows) === 1) {
            $obj = $this->array_to_object($rows[0], $class_name);
        } else {
            if ($error_off) {
                return false;
            } else {
                throw new Me_Exception(600);
            }
        }
        return $obj;
    }

    /**
     * INSERT
     * @param object $obj
     * @param array $target_column 処理対象カラム
     * @return boolean
     */
    public function insert($obj, $target_column = array())
    {
        // 作成・更新日時をセット
        $now = time();
        if (property_exists($obj, 'create_date')) {
            $obj->create_date = $now;
        }
        if (property_exists($obj, 'update_date')) {
            $obj->update_date = $now;
        }

        // 処理対象カラム
        // 指定がなければ全カラムを対象とする。
        if (empty($target_column)) {
            $target_column = $this->column_list;
        }

        // 対象カラム
        $set_column = array();
        foreach ($target_column as $col) {
            if ($col === '_id' && $obj->$col === '') {
                // _idが空文字の場合は、対象としない。
                continue;
            }
            if ($obj->$col !== null) {
                // null以外を処理対象とする。
                $set_column[] = $col;
            }
        }

        // INSERT文作成
        $query = 'INSERT INTO ' . $this->table_name . ' (';
        // 対象カラム追加
        $query .= join(', ', $set_column) . ")\n";
        $query .= 'VALUES ( ' . join(' , ', $set_column) . ' )';

        // SQL実行
        $sql = new Me_Sql();
        $sql->set_query($query);
        foreach ($set_column as $col) {
            $sql->bind('VALUES', $col, $obj->$col);
        }
        $sql->exec();

        // INSERTしたレコードの_idを取得
        $sql->set_query('SELECT last_insert_rowid() as _id');
        $rows = $sql->exec();
        $obj->_id = $rows[0]['_id'];

        return true;
    }

    /**
     * UPDATE
     * @param object $obj
     * @param array $target_column 処理対象カラム
     * @return boolean
     */
    public function update($obj, $target_column = array())
    {
        if (empty($obj->_id)) {
            // _idがなければfalseを返す。
            return false;
        }

        // 処理対象カラム
        // 指定がなければ全カラムを対象とする。
        if (empty($target_column)) {
            $target_column = $this->column_list;
        }

        // 更新日時をセット
        $now = time();
        if (property_exists($obj, 'update_date')) {
            $obj->update_date = $now;
            if (!in_array('update_date', $target_column)) {
                $target_column[] = 'update_date';
            }
        }

        // 対象カラム
        $set_column = array();
        foreach ($target_column as $key => $col) {
            if ($col === '_id') {
                // _idは、対象としない。
                unset($target_column[$key]);
                continue;
            }
            if ($obj->$col === null) {
                // nullは処理対象外
                unset($target_column[$key]);
            } else {
                // null以外を処理対象とする。
                $set_column[] = "{$col} = :{$col}";
            }
        }

        // UPDATE文作成
        $query = 'UPDATE ' . $this->table_name . "\nSET ";
        // 対象カラム追加
        $query .= join(' , ', $set_column);
        $query .= "\nWHERE _id = 1";

        // SQL実行
        $sql = new Me_Sql();
        $sql->set_query($query);
        foreach ($target_column as $col) {
            $sql->bind('SET', ':' . $col, $obj->$col);
        }
        $sql->bind('WHERE', '1', $obj->_id);
        $sql->exec();

        return true;
    }

    /**
     * DELETE
     * @param object $obj
     * @return boolean
     */
    public function delete($obj)
    {
        if (empty($obj->_id)) {
            // _idがなければfalseを返す。
            return false;
        }

        // DELETE文作成
        $query = 'DELETE FROM ' . $this->table_name . ' WHERE _id = :id';

        // SQL実行
        $sql = new Me_Sql();
        $sql->set_query($query);
        $sql->bind(0, 6, $obj->_id);
        $sql->exec();

        return true;
    }

    /**
     * INSERT or UPDATE
     * @param object $obj オブジェクト
     * @param array $target_column 処理対象カラム
     * @return boolean
     */
    public function insert_or_update($obj, $target_column = array())
    {
        if (empty($obj->_id)) {
            // _idが無ければ、新規登録
            return $this->insert($obj, $target_column);
        } else {
            // テンプレート_idがあれば、更新
            return $this->update($obj, $target_column);
        }
    }

    /**
     * 配列をオブジェクトに変換する
     * @param array $row
     * @param string $class_name クラス名
     * @return object $obj
     */
    public function array_to_object($row, $class_name = null)
    {
        if ($class_name !== null) {
            // クラス名が指定されている場合は、その名前のオブジェクトを生成する
            $obj = new $class_name;
        } else {
            // クラス名が渡されていない場合は、Entityオブジェクトを生成する
            $obj = new $this->class_name;
        }

        foreach ($this->column_list as $col) {
            // 配列の要素の中から、そのクラスが持っているプロパティだけをセットする
            if (isset($row[$col])) {
                $obj->$col = $row[$col];
            }
        }
        return $obj;
    }
}